/*------------------------------------------------------------------------------

Beetle01 classes Landscape and Square

The rectangular landscape of dimension XMAX x YMAX comprises a vector of
pointers to individual squares.
The landscape incorporates a function to produce a file of summary statistics
of the mean densities of resident and dispersing morphs in each square over a
specified number of generations.

Each square has a mean carrying capacity Ki and a current carrying capacity Kit
drawn from either a Poisson distribution with mean Ki or a normal distribution
with mean Ki and a specified temporal standard deviation.
Each square records the numbers of resident and dispersing morphs currently
present, plus running totals to enable mean density to be calculated over a
number of generations.

Author: Steve Palmer, University of Aberdeen

Last updated: 8 December 2011

------------------------------------------------------------------------------*/
#ifndef LandscapeH
#define LandscapeH

#include <vector>
#include <io.h>
#include <iostream.h>
#include <algorithm>
#include <stdlib.h>
#include <fstream.h>
#include <sstream>
#include "Parameters.h"
#include "stocc.h"
extern Parameters* parameters;
struct locn { int x; int y; };
struct sqStats { int K; double meanDisp; double meanRes; };
//string itos(int i) {	// convert int to string
//	stringstream s; s << i; return s.str();
//}

class Square
{
public:
  Square(int,int);  // default constructor
	locn get_locn(void);
//	int get_PreyDens(void);
	int get_Ki(void);
	int get_Kit(void);
	int get_PredDens(void);
	int get_Dispersers(void);
	int get_Residents(void);
	double get_PredDisp(void);
	void set_Ki(int,int,double);
	void update_Ki(int,double);
	void incrPredDens(int);
	void incrTotals(void);
	void resetPredStats(void);
  sqStats get_sqStats(void);

private:
	//input parameters
  locn sq;
  int Ki;
  int Kit;
//  int PredDens;
  int Dispersers; // current number of disperser morphs
  int Residents;  // current number of resident morphs
  int totDispersers; // running total of disperser morphs
  int totResidents;  // running total of resident morphs
  int Ngens; // number of generations contributing to running totals
};
class Landscape
{
public:
  Landscape(); // Landscape class constructor
  ~Landscape();
	int get_id(void);
	int get_Kmean(void);
	int get_Ktot(void);
	int get_Kmin(void);
	locn get_bounds(void);
	int get_min_K(void);
	void update_K(int,double);
	void incrTotals(void);
	void resetPredStats(void);
  Square* locate_square(int,int);
  int TotalPredN(void);
	void spatialSummary(int);
  locn randomSquare(double);
  locn attractiveSquare(double,int);

protected:
  int xmax; // Landscape size x )
  int ymax; // Landscape size y )
  std::vector <Square*> Squares; 

private:
  static int LandCounter; // Used to create ID, held by class not members of class
  int id;
  int Kmean; // total mean carrying capacity
  int Ktot;  // total realised carrying capacity
  int Kmin;  // minimum realised carrying capacity

};

//---------------------------------------------------------------------------
#endif
